@@ -498,14 +498,16 @@
                        (not (load fn)))))))))
 
 (defimplementation swank-compile-string (string &key buffer position filename
-                                         policy)
+                                                policy)
   (declare (ignore filename policy))
   (let ((jvm::*resignal-compiler-warnings* t)
         (*abcl-signaled-conditions* nil))
     (handler-bind ((warning #'handle-compiler-warning))
       (let ((*buffer-name* buffer)
             (*buffer-start-position* position)
-            (*buffer-string* string))
+            (*buffer-string* string)
+            (sys::*source* (make-pathname :device "emacs-buffer" :name buffer))
+            (sys::*source-position* position))
         (funcall (compile nil (read-from-string
                                (format nil "(~S () ~A)" 'lambda string))))
         t))))
@@ -540,13 +542,29 @@
 
 (defmethod source-location ((symbol symbol))
   (when (pathnamep (ext:source-pathname symbol))
-    (let ((pos (ext:source-file-position symbol)))
-      `(:location
-        (:file ,(namestring (ext:source-pathname symbol)))
-        ,(if pos
-             (list :position (1+ pos))
-             (list :function-name (string symbol)))
-        (:align t)))))
+    (let ((pos (ext:source-file-position symbol))
+          (path (namestring (ext:source-pathname symbol))))
+      (cond ((ext:pathname-jar-p path)
+             `(:location
+               ;; strip off "jar:file:" = 9 characters
+               (:zip ,@(split-string (subseq path 9) "!/"))
+               ;; pos never seems right. Use function name.
+               (:function-name ,(string symbol))
+               (:align t)))
+            ((equal (pathname-device (ext:source-pathname symbol)) "emacs-buffer")
+             ;; conspire with swank-compile-string to keep the buffer
+             ;; name in a pathname whose device is "emacs-buffer".
+             `(:location
+                (:buffer ,(pathname-name (ext:source-pathname symbol)))
+                (:function-name ,(string symbol))
+                (:align t)))
+            (t
+             `(:location
+                (:file ,path)
+                ,(if pos
+                     (list :position (1+ pos))
+                     (list :function-name (string symbol)))
+                (:align t)))))))
 
 (defmethod source-location ((frame sys::java-stack-frame))
   (destructuring-bind (&key class method file line) (sys:frame-to-list frame)
