@@ -161,6 +161,13 @@ form suitable for testing with #+."
    (and (find-package package)
         (find-symbol (string name) package))))
 
+(defun choose-symbol (package name alt-package alt-name)
+  "If symbol package:name exists return that symbol, otherwise alt-package:alt-name.
+  Suitable for use with #."
+  (or (and (find-package package)
+           (find-symbol (string name) package))
+      (find-symbol (string alt-name) alt-package)))
+
 
 ;;;; UFT8
 
@@ -611,7 +618,7 @@ Return nil if the file contains no special markers."
       (loop while (and (< p end)
                        (member (aref str p) '(#\space #\tab)))
             do (incf p))
-      (let ((end (position-if (lambda (c) (find c '(#\space #\tab #\newline)))
+      (let ((end (position-if (lambda (c) (find c '(#\space #\tab #\newline #\;)))
                               str :start p)))
         (find-external-format (subseq str p end))))))
 
@@ -719,7 +726,7 @@ return the results and T.  Otherwise, return the original form and
 NIL."
   (let ((fun (and (consp form)
                   (valid-function-name-p (car form))
-                  (compiler-macro-function (car form)))))
+                  (compiler-macro-function (car form) env))))
     (if fun
 	(let ((result (funcall *macroexpand-hook* fun form env)))
           (values result (not (eq result form))))
@@ -987,9 +994,10 @@ returns.")
 
 ;;;; Definition finding
 
-(defstruct (:location (:type list) :named
+(defstruct (location (:type list)
                       (:constructor make-location
-                                    (buffer position &optional hints)))
+                          (buffer position &optional hints)))
+  (type :location)
   buffer position
   ;; Hints is a property list optionally containing:
   ;;   :snippet SOURCE-TEXT
@@ -997,17 +1005,6 @@ returns.")
   ;;     the definition, which could be used in a text search.
   hints)
 
-(defstruct (:error (:type list) :named (:constructor)) message)
-
-;;; Valid content for BUFFER slot
-(defstruct (:file       (:type list) :named (:constructor)) name)
-(defstruct (:buffer     (:type list) :named (:constructor)) name)
-(defstruct (:etags-file (:type list) :named (:constructor)) filename)
-
-;;; Valid content for POSITION slot
-(defstruct (:position (:type list) :named (:constructor)) pos)
-(defstruct (:tag      (:type list) :named (:constructor)) tag1 tag2)
-
 (defmacro converting-errors-to-error-location (&body body)
   "Catches errors during BODY and converts them to an error location."
   (let ((gblock (gensym "CONVERTING-ERRORS+")))
@@ -1310,6 +1307,14 @@ Don't execute unwind-protected sections, don't raise conditions.
 (definterface receive-if (predicate &optional timeout)
   "Return the first message satisfiying PREDICATE.")
 
+(definterface wake-thread (thread)
+  "Trigger a call to CHECK-SLIME-INTERRUPTS in THREAD without using
+asynchronous interrupts."
+  (declare (ignore thread))
+  ;; Doesn't have to implement this if RECEIVE-IF periodically calls
+  ;; CHECK-SLIME-INTERRUPTS, but that's energy inefficient
+  nil)
+
 (definterface register-thread (name thread)
   "Associate the thread THREAD with the symbol NAME.
 The thread can then be retrieved with `find-registered'.
@@ -1397,6 +1402,27 @@ but that thread may hold it more than once."
   (declare (ignore hashtable))
   nil)
 
+
+;;;; Floating point
+
+(definterface float-nan-p (float)
+  "Return true if FLOAT is a NaN value (Not a Number)."
+  ;; When the float type implements IEEE-754 floats, two NaN values
+  ;; are never equal; when the implementation does not support NaN,
+  ;; the predicate should return false. An implementation can
+  ;; implement comparison with "unordered-signaling predicates", which
+  ;; emit floating point exceptions.
+  (handler-case (not (= float float))
+    ;; Comparisons never signal an exception other than the invalid
+    ;; operation exception (5.11 Details of comparison predicates).
+    (floating-point-invalid-operation () t)))
+
+(definterface float-infinity-p (float)
+  "Return true if FLOAT is positive or negative infinity."
+  (not (< most-negative-long-float
+          float
+          most-positive-long-float)))
+
 
 ;;;; Character names
 
