@@ -58,16 +58,18 @@
   "GRAY")
 
 
-;;;; TCP Server
+;;;; UTF8
 
-(defimplementation preferred-communication-style ()
-  ;; While ECL does provide threads, some parts of it are not
-  ;; thread-safe (2010-02-23), including the compiler and CLOS.
-  nil
-  ;; ECL on Windows does not provide condition-variables
-  ;; (or #+(and threads (not windows)) :spawn
-  ;;     nil)
-  )
+;;; Convert the string STRING to a (simple-array (unsigned-byte 8)).
+;;;
+;;;   string-to-utf8 (string)
+
+;;; Convert the (simple-array (unsigned-byte 8)) OCTETS to a string.
+;;;
+;;;   utf8-to-string (octets)
+
+
+;;;; TCP Server
 
 (defun resolve-hostname (name)
   (car (sb-bsd-sockets:host-ent-addresses
@@ -88,6 +90,12 @@
 (defimplementation close-socket (socket)
   (sb-bsd-sockets:socket-close socket))
 
+(defun accept (socket)
+  "Like socket-accept, but retry on EAGAIN."
+  (loop (handler-case
+            (return (sb-bsd-sockets:socket-accept socket))
+          (sb-bsd-sockets:interrupted-error ()))))
+
 (defimplementation accept-connection (socket
                                       &key external-format
                                       buffering timeout)
@@ -103,50 +111,74 @@
                                                        'character 
                                                        '(unsigned-byte 8))
                                      :external-format external-format))
-(defun accept (socket)
-  "Like socket-accept, but retry on EAGAIN."
-  (loop (handler-case
-            (return (sb-bsd-sockets:socket-accept socket))
-          (sb-bsd-sockets:interrupted-error ()))))
 
-(defimplementation socket-fd (socket)
-  (etypecase socket
-    (fixnum socket)
-    (two-way-stream (socket-fd (two-way-stream-input-stream socket)))
-    (sb-bsd-sockets:socket (sb-bsd-sockets:socket-file-descriptor socket))
-    (file-stream (si:file-stream-fd socket))))
+;;; Call FN whenever SOCKET is readable.
+;;;
+;;;   add-sigio-handler (socket fn)
 
-(defvar *external-format-to-coding-system*
-  '((:latin-1
-     "latin-1" "latin-1-unix" "iso-latin-1-unix" 
-     "iso-8859-1" "iso-8859-1-unix")
-    (:utf-8 "utf-8" "utf-8-unix")))
+;;; Remove all sigio handlers for SOCKET.
+;;;
+;;;   remove-sigio-handlers (socket)
 
-(defun external-format (coding-system)
-  (or (car (rassoc-if (lambda (x) (member coding-system x :test #'equal))
-                      *external-format-to-coding-system*))
-      (find coding-system (ext:all-encodings) :test #'string-equal)))
+;;; Call FN when Lisp is waiting for input and SOCKET is readable.
+;;;
+;;;   add-fd-handler (socket fn)
 
-(defimplementation find-external-format (coding-system)
-  #+unicode (external-format coding-system)
-  ;; Without unicode support, ECL uses the one-byte encoding of the
-  ;; underlying OS, and will barf on anything except :DEFAULT.  We
-  ;; return NIL here for known multibyte encodings, so
-  ;; SWANK:CREATE-SERVER will barf.
-  #-unicode (let ((xf (external-format coding-system)))
-              (if (member xf '(:utf-8))
-                  nil
-                  :default)))
+;;; Remove all fd-handlers for SOCKET.
+;;;
+;;;   remove-fd-handlers (socket)
+
+(defimplementation preferred-communication-style ()
+  (cond
+    ((member :threads *features*) :spawn)
+    ((member :windows *features*) nil)
+    (t #|:fd-handler|# nil)))
+
+;;; Set the 'stream 'timeout.  The timeout is either the real number
+;;; specifying the timeout in seconds or 'nil for no timeout.
+;;;
+;;;   set-stream-timeout (stream timeout)
+
+
+;;; Hook called when the first connection from Emacs is established.
+;;; Called from the INIT-FN of the socket server that accepts the
+;;; connection.
+;;;
+;;; This is intended for setting up extra context, e.g. to discover
+;;; that the calling thread is the one that interacts with Emacs.
+;;;
+;;;   emacs-connected ()
 
 
 ;;;; Unix Integration
 
+(defimplementation getpid ()
+  (si:getpid))
+
+;;; Call FUNCTION on SIGINT (instead of invoking the debugger).
+;;; Return old signal handler.
+;;;
+;;;   install-sigint-handler (function)
+
+;;; XXX!
 ;;; If ECL is built with thread support, it'll spawn a helper thread
 ;;; executing the SIGINT handler. We do not want to BREAK into that
 ;;; helper but into the main thread, though. This is coupled with the
 ;;; current choice of NIL as communication-style in so far as ECL's
 ;;; main-thread is also the Slime's REPL thread.
 
+(defun make-interrupt-handler (real-handler)
+  #+threads
+  (let ((main-thread (find 'si:top-level (mp:all-processes)
+                           :key #'mp:process-name)))
+    #'(lambda (&rest args)
+        (declare (ignore args))
+        (mp:interrupt-process main-thread real-handler)))
+  #-threads
+  #'(lambda (&rest args)
+      (declare (ignore args))
+      (funcall real-handler)))
+
 (defimplementation call-with-user-break-handler (real-handler function)
   (let ((old-handler #'si:terminal-interrupt))
     (setf (symbol-function 'si:terminal-interrupt)
@@ -154,75 +186,76 @@
     (unwind-protect (funcall function)
       (setf (symbol-function 'si:terminal-interrupt) old-handler))))
 
-#+threads
-(defun make-interrupt-handler (real-handler)
-  (let ((main-thread (find 'si:top-level (mp:all-processes)
-                           :key #'mp:process-name)))
-    #'(lambda (&rest args)
-        (declare (ignore args))
-        (mp:interrupt-process main-thread real-handler))))
+(defimplementation quit-lisp ()
+  (ext:quit))
 
-#-threads
-(defun make-interrupt-handler (real-handler)
-  #'(lambda (&rest args)
-      (declare (ignore args))
-      (funcall real-handler)))
+;;; Default implementation is fine.
+;;;
+;;;   lisp-implementation-type-name
+;;;   lisp-implementation-program
 
+(defimplementation socket-fd (socket)
+  (etypecase socket
+    (fixnum socket)
+    (two-way-stream (socket-fd (two-way-stream-input-stream socket)))
+    (sb-bsd-sockets:socket (sb-bsd-sockets:socket-file-descriptor socket))
+    (file-stream (si:file-stream-fd socket))))
 
-(defimplementation getpid ()
-  (si:getpid))
+;;; Create a character stream for the file descriptor FD. This
+;;; interface implementation requires either `ffi:c-inline' or has to
+;;; wait for the exported interface.
+;;;
+;;;   make-fd-stream (socket-stream)
 
-(defimplementation set-default-directory (directory)
-  (ext:chdir (namestring directory))  ; adapts *DEFAULT-PATHNAME-DEFAULTS*.
-  (default-directory))
+;;; Duplicate a file descriptor. If the syscall fails, signal a
+;;; condition. See dup(2). This interface requiers `ffi:c-inline' or
+;;; has to wait for the exported interface.
+;;;
+;;;   dup (fd)
+
+;;; Does not apply to ECL which doesn't dump images.
+;;;
+;;;   exec-image (image-file args)
+
+(defimplementation command-line-args ()
+  (ext:command-args))
+
+
+;;;; pathnames
+
+;;; Return a pathname for FILENAME.
+;;; A filename in Emacs may for example contain asterisks which should not
+;;; be translated to wildcards.
+;;;
+;;;   filename-to-pathname (filename)
+
+;;; Return the filename for PATHNAME.
+;;;
+;;;   pathname-to-filename (pathname)
 
 (defimplementation default-directory ()
   (namestring (ext:getcwd)))
 
-(defimplementation quit-lisp ()
-  (ext:quit))
+(defimplementation set-default-directory (directory)
+  (ext:chdir (namestring directory)) ; adapts *DEFAULT-PATHNAME-DEFAULTS*.
+  (default-directory))
 
 
+;;; Call FN with hooks to handle special syntax. Can we use it for
+;;; `ffi:c-inline' to be handled as C/C++ code?
+;;;
+;;;   call-with-syntax-hooks
 
-;;; Instead of busy waiting with communication-style NIL, use select()
-;;; on the sockets' streams.
-#+serve-event
-(progn
-  (defun poll-streams (streams timeout)
-    (let* ((serve-event::*descriptor-handlers*
-            (copy-list serve-event::*descriptor-handlers*))
-           (active-fds '())
-           (fd-stream-alist
-            (loop for s in streams
-                  for fd = (socket-fd s)
-                  collect (cons fd s)
-                  do (serve-event:add-fd-handler fd :input
-                                                 #'(lambda (fd)
-                                                     (push fd active-fds))))))
-      (serve-event:serve-event timeout)
-      (loop for fd in active-fds collect (cdr (assoc fd fd-stream-alist)))))
+;;; Return a suitable initial value for SWANK:*READTABLE-ALIST*.
+;;;
+;;;   default-readtable-alist
 
-  (defimplementation wait-for-input (streams &optional timeout)
-    (assert (member timeout '(nil t)))
-    (loop
-      (cond ((check-slime-interrupts) (return :interrupt))
-            (timeout (return (poll-streams streams 0)))
-            (t
-             (when-let (ready (poll-streams streams 0.2))
-               (return ready))))))  
+
+;;;; Packages
 
-) ; #+serve-event (progn ...
-
-#-serve-event
-(defimplementation wait-for-input (streams &optional timeout)
-  (assert (member timeout '(nil t)))
-  (loop
-   (cond ((check-slime-interrupts) (return :interrupt))
-         (timeout (return (remove-if-not #'listen streams)))
-         (t
-          (let ((ready (remove-if-not #'listen streams)))
-            (if ready (return ready))
-            (sleep 0.1))))))
+#+package-local-nicknames
+(defimplementation package-local-nicknames (package)
+  (ext:package-local-nicknames package))
 
 
 ;;;; Compilation
@@ -268,15 +301,6 @@
   (handler-bind ((c:compiler-message #'handle-compiler-message))
     (funcall function)))
 
-(defimplementation swank-compile-file (input-file output-file
-                                       load-p external-format
-                                       &key policy)
-  (declare (ignore policy))
-  (with-compilation-hooks ()
-    (compile-file input-file :output-file output-file
-                  :load load-p
-                  :external-format external-format)))
-
 (defvar *tmpfile-map* (make-hash-table :test #'equal))
 
 (defun note-buffer-tmpfile (tmp-file buffer-name)
@@ -288,8 +312,8 @@
 (defun tmpfile-to-buffer (tmp-file)
   (gethash tmp-file *tmpfile-map*))
 
-(defimplementation swank-compile-string (string &key buffer position filename
-                                                policy)
+(defimplementation swank-compile-string
+    (string &key buffer position filename policy)
   (declare (ignore policy))
   (with-compilation-hooks ()
     (let ((*buffer-name* buffer)        ; for compilation hooks
@@ -300,20 +324,65 @@
             (failure-p))
         (unwind-protect
              (with-open-file (tmp-stream tmp-file :direction :output
-                                                  :if-exists :supersede)
+                                         :if-exists :supersede)
                (write-string string tmp-stream)
                (finish-output tmp-stream)
                (multiple-value-setq (fasl-file warnings-p failure-p)
                  (compile-file tmp-file
-                   :load t
-                   :source-truename (or filename
-                                        (note-buffer-tmpfile tmp-file buffer))
-                   :source-offset (1- position))))
+                               :load t
+                               :source-truename (or filename
+                                                    (note-buffer-tmpfile tmp-file buffer))
+                               :source-offset (1- position))))
           (when (probe-file tmp-file)
             (delete-file tmp-file))
           (when fasl-file
             (delete-file fasl-file)))
         (not failure-p)))))
+
+(defimplementation swank-compile-file (input-file output-file
+                                       load-p external-format
+                                       &key policy)
+  (declare (ignore policy))
+  (with-compilation-hooks ()
+    (compile-file input-file :output-file output-file
+                  :load load-p
+                  :external-format external-format)))
+
+(defvar *external-format-to-coding-system*
+  '((:latin-1
+     "latin-1" "latin-1-unix" "iso-latin-1-unix"
+     "iso-8859-1" "iso-8859-1-unix")
+    (:utf-8 "utf-8" "utf-8-unix")))
+
+(defun external-format (coding-system)
+  (or (car (rassoc-if (lambda (x) (member coding-system x :test #'equal))
+                      *external-format-to-coding-system*))
+      (find coding-system (ext:all-encodings) :test #'string-equal)))
+
+(defimplementation find-external-format (coding-system)
+  #+unicode (external-format coding-system)
+  ;; Without unicode support, ECL uses the one-byte encoding of the
+  ;; underlying OS, and will barf on anything except :DEFAULT.  We
+  ;; return NIL here for known multibyte encodings, so
+  ;; SWANK:CREATE-SERVER will barf.
+  #-unicode (let ((xf (external-format coding-system)))
+              (if (member xf '(:utf-8))
+                  nil
+                  :default)))
+
+
+;;; Default implementation is fine
+;;;
+;;;   guess-external-format
+
+
+;;;; Streams
+
+;;; Implemented in `gray'
+;;;
+;;;   make-output-stream
+;;;   make-input-stream
+
 
 ;;;; Documentation
 
@@ -322,14 +391,27 @@
       (ext:function-lambda-list name)
     (if foundp arglist :not-available)))
 
+(defimplementation type-specifier-p (symbol)
+  (or (subtypep nil symbol)
+      (not (eq (type-specifier-arglist symbol) :not-available))))
+
 (defimplementation function-name (f)
   (typecase f
     (generic-function (clos:generic-function-name f))
     (function (si:compiled-function-name f))))
 
-;; FIXME
-;; (defimplementation macroexpand-all (form &optional env)
-;; (declare (ignore env))
+;;; Default implementation is fine (CL).
+;;; 
+;;; valid-function-name-p (form)
+
+#+walker
+(defimplementation macroexpand-all (form &optional env)
+  (walker:macroexpand-all form env))
+
+;;; Default implementation is fine.
+;;;
+;;;   compiler-macroexpand-1
+;;;   compiler-macroexpand
 
 (defimplementation collect-macro-forms (form &optional env)
   ;; Currently detects only normal macros, not compiler macros.
@@ -340,6 +422,11 @@
         (compile nil `(lambda () ,form))))
     (values macro-forms nil)))
 
+;;; Expand the format string CONTROL-STRING.
+;;; Default implementation is fine.
+;;;
+;;;   format-string-expand
+
 (defimplementation describe-symbol-for-emacs (symbol)
   (let ((result '()))
     (flet ((frob (type boundp)
@@ -358,12 +445,8 @@
     (:class (documentation name 'class))
     (t nil)))
 
-(defimplementation type-specifier-p (symbol)
-  (or (subtypep nil symbol)
-      (not (eq (type-specifier-arglist symbol) :not-available))))
-
 
-;;; Debugging
+;;;; Debugging
 
 (eval-when (:compile-toplevel :load-toplevel :execute)
   (import
@@ -403,37 +486,32 @@
 
 (defvar *backtrace* '())
 
-;;; Commented out; it's not clear this is a good way of doing it. In
-;;; particular because it makes errors stemming from this file harder
-;;; to debug, and given the "young" age of ECL's swank backend, that's
-;;; a bad idea.
+(defun in-swank-package-p (x)
+  (and
+   (symbolp x)
+   (member (symbol-package x)
+           (list #.(find-package :swank)
+                 #.(find-package :swank/backend)
+                 #.(ignore-errors (find-package :swank-mop))
+                 #.(ignore-errors (find-package :swank-loader))))
+   t))
 
-;; (defun in-swank-package-p (x)
-;;   (and
-;;    (symbolp x)
-;;    (member (symbol-package x)
-;;            (list #.(find-package :swank)
-;;                  #.(find-package :swank/backend)
-;;                  #.(ignore-errors (find-package :swank-mop))
-;;                  #.(ignore-errors (find-package :swank-loader))))
-;;    t))
+(defun is-swank-source-p (name)
+  (setf name (pathname name))
+  (pathname-match-p
+   name
+   (make-pathname :defaults swank-loader::*source-directory*
+                  :name (pathname-name name)
+                  :type (pathname-type name)
+                  :version (pathname-version name))))
 
-;; (defun is-swank-source-p (name)
-;;   (setf name (pathname name))
-;;   (pathname-match-p
-;;    name
-;;    (make-pathname :defaults swank-loader::*source-directory*
-;;                   :name (pathname-name name)
-;;                   :type (pathname-type name)
-;;                   :version (pathname-version name))))
-
-;; (defun is-ignorable-fun-p (x)
-;;   (or
-;;    (in-swank-package-p (frame-name x))
-;;    (multiple-value-bind (file position)
-;;        (ignore-errors (si::bc-file (car x)))
-;;      (declare (ignore position))
-;;      (if file (is-swank-source-p file)))))
+(defun is-ignorable-fun-p (x)
+  (or
+   (in-swank-package-p (frame-name x))
+   (multiple-value-bind (file position)
+       (ignore-errors (si::bc-file (car x)))
+     (declare (ignore position))
+     (if file (is-swank-source-p file)))))
 
 (defimplementation call-with-debugging-environment (debugger-loop-fn)
   (declare (type function debugger-loop-fn))
@@ -454,7 +532,7 @@
                         (name (si::frs-tag f)))
                    (unless (si::fixnump name)
                      (push name (third x)))))))
-    (setf *backtrace* (nreverse *backtrace*))
+    (setf *backtrace* (remove-if #'is-ignorable-fun-p (nreverse *backtrace*)))
     (set-break-env)
     (set-current-ihs)
     (let ((*ihs-base* *ihs-top*))
@@ -508,6 +586,11 @@
 (defimplementation print-frame (frame stream)
   (format stream "~A" (first frame)))
 
+;;; Is the frame FRAME restartable?.
+;;; Return T if `restart-frame' can safely be called on the frame.
+;;;
+;;; frame-restartable-p (frame)
+
 (defimplementation frame-source-location (frame-number)
   (nth-value 1 (frame-function (elt *backtrace* frame-number))))
 
@@ -535,22 +618,28 @@
   (let ((env (second (elt *backtrace* frame-number))))
     (si:eval-with-env form env)))
 
+;;; frame-package
+;;; frame-call
+;;; return-from-frame
+;;; restart-frame
+;;; print-condition
+;;; condition-extras
+
 (defimplementation gdb-initial-commands ()
   ;; These signals are used by the GC.
   #+linux '("handle SIGPWR  noprint nostop"
             "handle SIGXCPU noprint nostop"))
 
-(defimplementation command-line-args ()
-  (loop for n from 0 below (si:argc) collect (si:argv n)))
+;;; active-stepping
+;;; sldb-break-on-return
+;;; sldb-break-at-start
+;;; sldb-stepper-condition-p
+;;; sldb-setp-into
+;;; sldb-step-next
+;;; sldb-step-out
 
 
-;;;; Inspector
-
-;;; FIXME: Would be nice if it was possible to inspect objects
-;;; implemented in C.
-
-
-;;;; Definitions
+;;;; Definition finding
 
 (defvar +TAGS+ (namestring
                 (merge-pathnames "TAGS" (translate-logical-pathname "SYS:"))))
@@ -698,9 +787,26 @@
   (or (source-location object)
       (make-error-location "Source definition of ~S not found." object)))
 
+;;; buffer-first-change
+
+
+;;;; XREF
+
+;;; who-calls
+;;; calls-who
+;;; who-references
+;;; who-binds
+;;; who-sets
+;;; who-macroexpands
+;;; who-specializes
+;;; list-callers
+;;; list-callees
+
 
 ;;;; Profiling
 
+;;; XXX: use monitor.lisp (ccl,clisp)
+
 #+profile
 (progn
 
@@ -730,7 +836,37 @@
 ) ; #+profile (progn ...
 
 
-;;;; Threads
+;;;; Trace
+
+;;; Toggle tracing of the function(s) given with SPEC.
+;;; SPEC can be:
+;;;  (setf NAME)                            ; a setf function
+;;;  (:defmethod NAME QUALIFIER... (SPECIALIZER...)) ; a specific method
+;;;  (:defgeneric NAME)                     ; a generic function with all methods
+;;;  (:call CALLER CALLEE)                  ; trace calls from CALLER to CALLEE.
+;;;  (:labels TOPLEVEL LOCAL)
+;;;  (:flet TOPLEVEL LOCAL) 
+;;;
+;;;   toggle-trace (spec)
+
+
+;;;; Inspector
+
+;;; FIXME: Would be nice if it was possible to inspect objects
+;;; implemented in C.
+
+;;; Return a list of bindings corresponding to OBJECT's slots.
+;;;   eval-context (object)
+
+;;; Return a string describing the primitive type of object.
+;;;   describe-primitive-type (object)
+
+
+;;;; Multithreading
+
+;;; Not needed in ECL
+;;;
+;;;   initialize-multiprocessing
 
 #+threads
 (progn
@@ -777,12 +913,7 @@
         "RUNNING"
         "STOPPED"))
 
-  (defimplementation make-lock (&key name)
-    (mp:make-lock :name name :recursive t))
-
-  (defimplementation call-with-lock-held (lock function)
-    (declare (type function function))
-    (mp:with-lock (lock) (funcall function)))
+  ;; thread-attributes
 
   (defimplementation current-thread ()
     mp:*current-process*)
@@ -790,15 +921,15 @@
   (defimplementation all-threads ()
     (mp:all-processes))
 
+  (defimplementation thread-alive-p (thread)
+    (mp:process-active-p thread))
+
   (defimplementation interrupt-thread (thread fn)
     (mp:interrupt-process thread fn))
 
   (defimplementation kill-thread (thread)
     (mp:process-kill thread))
 
-  (defimplementation thread-alive-p (thread)
-    (mp:process-active-p thread))
-
   (defvar *mailbox-lock* (mp:make-lock :name "mailbox lock"))
   (defvar *mailboxes* (list))
   (declaim (type list *mailboxes*))
@@ -825,6 +956,8 @@
               (nconc (mailbox.queue mbox) (list message)))
         (mp:condition-variable-broadcast (mailbox.cvar mbox)))))
 
+  ;; receive
+
   (defimplementation receive-if (test &optional timeout)
     (let* ((mbox (mailbox (current-thread)))
            (mutex (mailbox.mutex mbox)))
@@ -838,8 +971,124 @@
                (setf (mailbox.queue mbox) (nconc (ldiff q tail) (cdr tail)))
                (return (car tail))))
            (when (eq timeout t) (return (values nil t)))
-           (mp:condition-variable-timedwait (mailbox.cvar mbox)
-                                            mutex
-                                            0.2)))))
+           (mp:condition-variable-wait (mailbox.cvar mbox) mutex)))))
 
-  ) ; #+threads (progn ...
+  ;; Trigger a call to CHECK-SLIME-INTERRUPTS in THREAD without using
+  ;; asynchronous interrupts.
+  ;;
+  ;; Doesn't have to implement this if RECEIVE-IF periodically calls
+  ;; CHECK-SLIME-INTERRUPTS, but that's energy inefficient.
+  ;;
+  ;;   wake-thread (thread)
+
+  ;; Copied from sbcl.lisp and adjusted to ECL.
+  (let ((alist '())
+        (mutex (mp:make-lock :name "register-thread")))
+
+    (defimplementation register-thread (name thread)
+      (declare (type symbol name))
+      (mp:with-lock (mutex)
+        (etypecase thread
+          (null
+           (setf alist (delete name alist :key #'car)))
+          (mp:process
+           (let ((probe (assoc name alist)))
+             (cond (probe (setf (cdr probe) thread))
+                   (t (setf alist (acons name thread alist))))))))
+      nil)
+
+    (defimplementation find-registered (name)
+      (mp:with-lock (mutex)
+        (cdr (assoc name alist)))))
+
+  ;; Not needed in ECL (?).
+  ;;
+  ;;   set-default-initial-binding (var form)
+
+  ) ; #+threads
+
+;;; Instead of busy waiting with communication-style NIL, use select()
+;;; on the sockets' streams.
+#+serve-event
+(defimplementation wait-for-input (streams &optional timeout)
+  (assert (member timeout '(nil t)))
+  (flet ((poll-streams (streams timeout)
+           (let* ((serve-event::*descriptor-handlers*
+                   (copy-list serve-event::*descriptor-handlers*))
+                  (active-fds '())
+                  (fd-stream-alist
+                   (loop for s in streams
+                      for fd = (socket-fd s)
+                      collect (cons fd s)
+                      do (serve-event:add-fd-handler fd :input
+                                                     #'(lambda (fd)
+                                                         (push fd active-fds))))))
+             (serve-event:serve-event timeout)
+             (loop for fd in active-fds collect (cdr (assoc fd fd-stream-alist))))))
+    (loop
+       (cond ((check-slime-interrupts) (return :interrupt))
+             (timeout (return (poll-streams streams 0)))
+             (t
+              (when-let (ready (poll-streams streams 0.2))
+                (return ready)))))))
+
+#-serve-event
+(defimplementation wait-for-input (streams &optional timeout)
+  (assert (member timeout '(nil t)))
+  (loop
+   (cond ((check-slime-interrupts) (return :interrupt))
+         (timeout (return (remove-if-not #'listen streams)))
+         (t
+          (let ((ready (remove-if-not #'listen streams)))
+            (if ready (return ready))
+            (sleep 0.1))))))
+
+
+;;;; Locks
+
+#+threads
+(defimplementation make-lock (&key name)
+  (mp:make-lock :name name :recursive t))
+
+(defimplementation call-with-lock-held (lock function)
+  (declare (type function function))
+  (mp:with-lock (lock) (funcall function)))
+
+
+;;;; Weak datastructures
+
+#+ecl-weak-hash
+(progn
+  (defimplementation make-weak-key-hash-table (&rest args)
+    (apply #'make-hash-table :weakness :key args))
+
+  (defimplementation make-weak-value-hash-table (&rest args)
+    (apply #'make-hash-table :weakness :value args))
+
+  (defimplementation hash-table-weakness (hashtable)
+    (ext:hash-table-weakness hashtable)))
+
+
+;;;; Character names
+
+;;; Default implementation is fine.
+;;;
+;;;   character-completion-set (prefix matchp)
+
+
+;;;; Heap dumps
+
+;;; Doesn't apply to ECL.
+;;;
+;;;   save-image (filename &optional restart-function)
+;;;   background-save-image (filename &key restart-function completion-function)
+
+
+;;;; Wrapping
+
+;;; Intercept future calls to SPEC and surround them in callbacks.
+;;; Very much similar to so-called advices for normal functions.
+;;;
+;;;   wrap (spec indicator &key before after replace)
+;;;   unwrap (spec indicator)
+;;;   wrapped-p (spec indicator)
